@@ -21,6 +21,21 @@
 
 #include "quick-launch.h"
 
+static gboolean
+filter_func                                     (GtkTreeModel *model,
+                                                 GtkTreeIter  *iter,
+                                                 gchar        *text,
+                                                 gpointer      data)
+{
+    gboolean retvalue;
+    gchar *rowtext = NULL;
+
+    gtk_tree_model_get (model, iter, 0, &rowtext, -1);
+    retvalue = g_str_has_prefix (rowtext, text);
+    g_free (rowtext);
+
+    return retvalue;
+}
 
 void
 button_toggled ( GtkWidget *wdg, gpointer user_data )
@@ -90,16 +105,18 @@ save_info(GtkWidget *wid, gpointer user_data)
 
         for(i=0;i<4;i++) {
 		        
-		lst = g_slist_append (lst, names[i]);
+		if ( names[i] != NULL ) lst = g_slist_append (lst, names[i]);
+		if ( icons[i] != NULL ) {
 		info = gtk_icon_theme_lookup_icon (it, g_strdup_printf(icons[i]), 64, 0);
 		if (info)
 			icons[i] = g_strdup (gtk_icon_info_get_filename (info));
 		
 		lst = g_slist_append (lst, icons[i]);
-		lst = g_slist_append (lst, ossos[i]);
-		lst = g_slist_append (lst, bins[i]);        
+		}
+		if ( ossos[i] != NULL ) lst = g_slist_append (lst, ossos[i]);
+		if ( bins[i] != NULL ) lst = g_slist_append (lst, bins[i]);        
 
-		gconf_client_set_list ( client, g_strdup_printf("%s%s",APP_GCONF_KEY,group_name[i]), GCONF_VALUE_STRING, lst, &error);
+		if ( lst != NULL ) gconf_client_set_list ( client, g_strdup_printf("%s%s",APP_GCONF_KEY,group_name[i]), GCONF_VALUE_STRING, lst, &error);
 		lst = NULL;
 	}
 	
@@ -133,6 +150,7 @@ parse_settings(GConfClient* client)
 	int i;
 	GSList *lst = NULL;
 	gpointer *nm,*in,*o,*b;
+	GdkPixbuf *pixim;
 	
 	for(i=0;i<4;i++) {
 		if (gconf_entry_get_value (g_strdup_printf("%s%s",APP_GCONF_KEY,group_name[i])) != NULL) {
@@ -147,7 +165,9 @@ parse_settings(GConfClient* client)
 				ossos[i] = (gchar *)o;
 				bins[i] = (gchar *)b;
 				if ( g_strcmp0(in,"") != 0 ) {
-					icoim = gtk_image_new_from_file( (gchar *)in );
+					pixim = gdk_pixbuf_new_from_file ( (gchar *)in, NULL );
+					icoim = gtk_image_new_from_pixbuf (gdk_pixbuf_scale_simple ( pixim, 48,48, GDK_INTERP_BILINEAR ));
+				
 					hildon_button_set_image ( HILDON_BUTTON(btns[i]), icoim );
 					hildon_button_set_text ( HILDON_BUTTON(btns[i]), g_strreplace((gchar *)nm," ","\n")  , "Tap to set");
 				}
@@ -308,7 +328,7 @@ add_apps( HildonTouchSelector *btn, gpointer p)
          
 	hildon_touch_selector_set_live_search ( HILDON_TOUCH_SELECTOR(btn), TRUE );
         
-        
+       /* 
  	GtkTreeModel *m;
  	GtkTreeSortable *sortable;
  	       
@@ -316,7 +336,7 @@ add_apps( HildonTouchSelector *btn, gpointer p)
         sortable = GTK_TREE_SORTABLE(m);
 	gtk_tree_sortable_set_sort_func(sortable, NULL, sort_iter_compare_func, NULL,NULL );
 	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(sortable), 0, GTK_SORT_ASCENDING);
-	hildon_touch_selector_set_model ( HILDON_TOUCH_SELECTOR(btn), 0, GTK_TREE_MODEL(sortable) );
+	hildon_touch_selector_set_model ( HILDON_TOUCH_SELECTOR(btn), 0, GTK_TREE_MODEL(sortable) );*/
 	GtkWidget *tmp;
 	GtkWidget *b;
 	
@@ -327,7 +347,13 @@ add_apps( HildonTouchSelector *btn, gpointer p)
 		gtk_box_pack_start ( GTK_BOX(tmp), b, FALSE,TRUE, 1);
 		g_signal_connect ( G_OBJECT(b), "clicked", G_CALLBACK(free_slot), (gpointer)p ); 
 		gtk_widget_show_all ( GTK_WIDGET(tmp) );
-	}	
+	}
+	
+	GtkTreeModelFilter *filter;
+	
+	filter = GTK_TREE_MODEL_FILTER (gtk_tree_view_get_model (GTK_TREE_VIEW (btn)));
+	hildon_live_search_set_filter (btn, filter);
+	hildon_live_search_set_visible_func (btn, filter_func, NULL, NULL);	
 
 
 }
